'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

function createHoc(hook) {
    return (map) => (component) => {
        const hoc = (props) => {
            const [state, actions] = hook();
            return react.createElement(component, Object.assign(Object.assign({}, props), map(state, actions)));
        };
        hoc.displayName = `WithDakpan(${component.displayName || component.name || 'Component'})`;
        return hoc;
    };
}

function createHook(context) {
    return (nullable) => {
        const state = react.useContext(context);
        if (!state && !nullable) {
            throw Error('Provider not mounted');
        }
        return [
            state && state.state,
            state && state.actions
        ];
    };
}

function createProvider(context, actions, state) {
    return ({ children, value, reinitialize = false }) => {
        const [, update] = react.useState({});
        const initialRef = react.useRef(value);
        const stateRef = react.useRef(state || value);
        const mapped = react.useMemo(() => Object.keys(actions).reduce((previous, current) => {
            const execute = (...args) => () => Promise.resolve(actions[current](...args)(stateRef.current, () => stateRef.current))
                .then((next) => {
                if (next) {
                    stateRef.current = next;
                    update({});
                }
            });
            const action = (...args) => execute(...args)();
            action.c = execute;
            return Object.assign(Object.assign({}, previous), { [current]: action });
        }, {}), [actions, stateRef]);
        react.useEffect(() => {
            if (reinitialize && value !== initialRef.current) {
                stateRef.current = initialRef.current = value;
                update({});
            }
        }, [value, reinitialize, initialRef]);
        return react.createElement(context.Provider, {
            value: {
                state: stateRef.current,
                actions: mapped
            }
        }, children);
    };
}

function createDakpan(initialState) {
    return (actions) => {
        const context = react.createContext(undefined);
        const hook = createHook(context);
        const state = initialState instanceof Function ? initialState() : initialState;
        return [
            createProvider(context, actions, state),
            hook,
            createHoc(hook)
        ];
    };
}

exports.createDakpan = createDakpan;
//# sourceMappingURL=index.js.map
